//Source & infos: https://homofaciens.de/technics-miscellaneous-electric-generator_en.htm
//compile with: gcc 4-stroke-generator-raspberry.c -o 4-stroke-generator-raspberry -I/usr/local/include -L/usr/local/lib -lwiringPi -lm -lwiringPiDev


#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include <unistd.h>
#include <string.h>
#include <time.h>

#include <wiringPi.h>
#include <lcd.h>

#ifndef TRUE
#  define TRUE  (1==1)
#  define FALSE (1==2)
#endif

#define LCD_RS     15
#define LCD_EN     16
#define LCD_D0      1
#define LCD_D1      6
#define LCD_D2     10
#define LCD_D3     11

#define LCD_ROWS    2
#define LCD_COLS   16
#define LCD_BITS    4

#define HALL_SENSOR 4


//Global lcd handle:
static int lcdHandle ;

void sensorTriggered(double bounceLoop) {
  double triggerPause = 0;

  triggerPause = 0;
  while(triggerPause < bounceLoop){
    if(digitalRead(HALL_SENSOR) == 1){
      triggerPause++;
    }
    else{
      triggerPause = 0;
    }
    usleep(10);
  }

  triggerPause = 0;
  while(triggerPause < bounceLoop){
    if(digitalRead(HALL_SENSOR) == 0){
      triggerPause++;
    }
    else{
      triggerPause = 0;
    }
    usleep(10);
  }

}

long get_rev_speed(long wait_time){
  long rpm = 0;
  long pulses=0;
  time_t timeStart, timeNow = 0;

  timeStart = time(NULL);

  while(timeNow-timeStart < wait_time){
    sensorTriggered(10);
    pulses++;
    timeNow = time(NULL);
    //printf("timeNow-timeStart=%ld\n", timeNow-timeStart);
  }

  rpm = pulses * 60 / wait_time;

  return rpm;
}


//######################################################################
//######################################################################
//######################################################################
int main (int argc, char *argv[]){
  int i;
  int lcd;

  struct tm *t;
  time_t tim;

  char buf[32];
  long rpm=0;
  char LCD_line_1[20];
  char LCD_line_2[20];


  wiringPiSetup();

  lcdHandle = lcdInit(LCD_ROWS, LCD_COLS, LCD_BITS, LCD_RS, LCD_EN, LCD_D0, LCD_D1, LCD_D2, LCD_D3,0,0,0,0);

  if(lcdHandle < 0){
    fprintf(stderr, "%s: lcdInit failed\n", argv [0]);
    return -1;
  }


  lcdPosition(lcdHandle, 0, 0);
  lcdPuts (lcdHandle, "  Raspberry Pi  ") ;
  lcdPosition(lcdHandle, 0, 1);
  lcdPuts (lcdHandle, "                ") ;
  sleep(2);

  while(1){

    rpm = get_rev_speed(3);
    lcdPosition(lcdHandle, 0, 0);
    lcdPuts (lcdHandle, "  Raspberry Pi  ") ;
    sprintf(LCD_line_2, "Crank =% 6drpm",  rpm);
    lcdPosition(lcdHandle, 0, 1);
    lcdPuts (lcdHandle, LCD_line_2);
  }

  printf("Exiting program!?!\n");

  return 0 ;
}
